package ThumbIM;
use strict;
use warnings;
use Carp;

sub new {
  my $this = shift;
  my $class = ref($this) || $this;
  my $self = {};
  $self->{img} = Image::Magick->new;
  bless $self, $class;
}

sub destroy {
  @{$_[0]->{img}} = ();
}

sub setsize {
  my ($self, $x, $y) = @_;
  $self->{img}->Set(size=>joinx($x, $y));
  $self->{img}->Set(page=>'0x0+0+0');
}

sub create {
  my ($self, $out) = @_;
  croak "Unable to write $out" if $self->{img}->Write(filename=>"PNG:$out");
}

sub setcolour {
  "#$_[1]";
}

sub stone {
  my ($self, $sf, $st, $dooutline, $x, $y, $fgcolour, $bgcolour) = @_;
  my $outline = $sf * $st/2;
  my $sw = $dooutline ? $st/20 : 0;
  my $string = join(',', $x, $y, $outline, $outline, 0, 360);
  $self->{img}->Draw(primitive=>'Ellipse',stroke=>$fgcolour,fill=>$bgcolour,strokewidth=>$sw,points=>$string);
}

sub stonef {
  my ($self, $x, $y, $colour, $factor, $st, $fb, $path) = @_;
  $self->{img}->Annotate(font=>$path,pointsize=>$factor,fill=>$colour,text=>chr($fb),x=>$x - $st/2,'y'=>$y + $st/2);
}

sub dots {
  my ($self, $st, $x, $y, $linecolour) = @_;
  my $string = join(',', $x, $y, $st/30, $st/30, 0, 360);
  $self->{img}->Draw(primitive=>'Ellipse',stroke=>$linecolour,fill=>$linecolour,strokewidth=>$st/20,points=>$string);
}

sub board {
  my ($self, $writer, $x, $y, $bo, @p) = @_;
  $self->{img}->ReadImage("canvas:$writer->{colours}->{offboard}");
  $self->{img}->Draw(primitive=>'Rectangle',fill=>$writer->{colours}->{board},antialias=>0,points=>join ',', @p);
}

sub square {
  my ($self, $colour, @p) = @_;
  $self->{img}->Draw(primitive=>'Rectangle',fill=>$colour,antialias=>0,points=>join ',', @p);
}

sub rotate {
  my ($self) = @_;
  $self->{img}->Rotate(degrees=>180);
}

sub line {
  my ($self, $colour, $halfstone, @p) = @_;
  $self->{img}->Draw(primitive=>'Line',fill=>$colour,stroke=>$colour,strokewidth=>$halfstone/20,antialias=>0,points=>join ',', @p);
}

sub crop {
  my ($self, $geowidth, $geoheight, $minx, $miny) = @_;
  $self->{img}->Crop(geometry => joinx($geowidth + 1, $geoheight + 1, $minx, $miny));
  $self->{img}->Set(page=>'0x0+0+0');
}

sub resize {
  my ($self, $scale) = @_;
  $self->{img}->Resize(geometry=>joinx($scale, $scale, 0, 0));
}

sub frame {
  my ($self, $writer, $x, $y) = @_;
  $self->{img}->Set(mattecolor=>$writer->{colours}->{offboard});
  my $extension = $writer->{stonesize} * $writer->{extension};
  $self->{img}->Frame(joinx($extension, $extension));
}

sub joinx {
  my ($sizex, $sizey, $a, $b) = @_;
  my $xstring = "${sizex}x$sizey";
  $xstring .= "+$a+$b" if defined $a && defined $b;
  return $xstring;
}

1;
